@@ -1,4 +1,4 @@
-from arklibrary import Stream, Header, Ini
+from arklibrary import Stream, Ini
 import socket
 from pathlib import Path
 
@@ -70,124 +70,89 @@ class GameBotClient:
     """
 
     PATH = Path.cwd() / Path('config.ini')
-    __ID = 0
 
-    def __init__(self, type: str = None, timeout=5):
-        self.type = type
-        self.__ID = GameBotClient.__ID
-        GameBotClient.__ID += 1
-        message = f'Socket {self.__ID} Connection Started'.center(100)
-        print(f'\r\033[7;35m{message}\033[0m')
+    def __init__(self, server_id=None, host=None, port=None, timeout=5):
         user_config = Path().cwd() / Path('config.ini')
         default_config = Path(__file__).parent / Path('config.ini')
         config = Ini(user_config) if user_config.exists() else Ini(default_config)
-        self.server_host = config['NETWORK']['host']
-        self.server_port = int(config['NETWORK']['port'])
-        self.client_host = config['SERVER']['host']
-        self.client_port = config['SERVER']['port']
+        self.server_id = server_id
+        self.host = host or config['NETWORK']['host'] or '127.0.0.1'
+        self.port = host or int(config['NETWORK']['port']) or 65432
+        self.server_address = f"{self.host}:{self.port}"
+        self.client_address = None
         self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self.socket.settimeout(timeout)
         self.connected = False
         self.timeout = None
         self.header = None
 
-    def connect(self, host=None, port=None):
-        connecting(f"Connecting to {self.server_host}:{self.server_port}")
+    def connect(self):
+        connecting(f"Connecting to {self.server_address}")
         try:
-            self.socket.connect((host or self.server_host, port or self.server_port))
+            self.socket.connect((self.host, self.port))
             self.connected = True
-            connect(f'{self.server_host}:{self.server_port}')
-            if self.type == "bot":
-                self.register_bot()
+            connect(f'{self.server_address}')
+            out_stream = Stream(type='client', function='connect', from_address=self.client_address, to_address=self.server_address)
+            self.socket.sendall(out_stream.encode())
+            in_stream = self.read()
+            self.client_address = f"{in_stream.to_address}"
+
         except ConnectionRefusedError:
-            connect(f"Unable to connect with {self.server_host}:{self.server_port}", failed=True)
+            connect(f"Unable to connect with {self.server_address}", failed=True)
             self.connected = False
 
-    def register_bot(self):
-        data = {
-            'server_id': '0',
-            'server_name': 'My Server',
-            'map': 'Ragnarok',
-            'to_address': '127.0.0.1:65432',
-            'encoding': 'Stream',
-            'registering': True,
-            'type': 'bot'
-        }
-        self.send("register", **data)
-
-    def bot_header(self):
-        return {
-            'server_id': '0',
-            'server_name': 'My Server',
-            'map': 'Ragnarok',
-            'to_address': '127.0.0.1:65432',
-            'encoding': 'Stream',
-            'pinging': True,
-            'type': 'bot'
-        }
-
-    def api_header(self):
-        return {
-            'server_id': '0',
-            'server_name': 'My Server',
-            'map': 'Ragnarok',
-            'to_address': '127.0.0.1:65432',
-            'encoding': 'Stream',
-            'type': 'api'
-        }
-
     def disconnect(self):
         if not self.connected:
             return
-        disconnecting(f"{self.server_host}:{self.server_port}")
+        disconnecting(f"{self.server_address}")
         self.socket.close()
         self.socket = None
         self.connected = False
-        disconnect(f"{self.server_host}:{self.server_port}")
-
-    def read_header(self):
-        if not self.connected:
-            return
-        dimension_stream: bytes = self.socket.recv(Header.DIMENSION)
-        header_length: int = Header.dimension(dimension_stream)
-        header_stream = self.socket.recv(header_length)
-        header = Header.decode(header_stream)
-        header_length = Header.dimension(dimension)
-        return Header.header_stream(self.socket.recv(header_length))
+        disconnect(f"{self.server_address}")
 
     def readall(self):
         if not self.connected:
             return
-        header = self.read_header()
         fragments = bytearray()
         chunk_size = 40  # number of bytes to read at a time
+        size = self.socket.recv(Stream.SIZE)
+        if not size:
+            return
+        message_length = int(size.decode())  # TODO: disconnect if not possible because its someone trying to hack
         while len(fragments) < message_length:
             chunk = bytearray(self.socket.recv(chunk_size))  # Should be ready to read
             fragments += chunk
         recv_data = bytes(fragments)
         return recv_data
 
-    def read(self, num_bytes: int = None) -> bytes:
+    def read(self, num_bytes: int = None) -> Stream:
         if num_bytes:
             recv_data = self.socket.recv(num_bytes)
         else:
             recv_data = self.readall()
-        stream = Stream(stream=recv_data)
-        response(stream.decode())
-        return recv_data
+        stream = Stream.new(recv_data)
+        response(stream.body)
+        return stream
 
-    def send(self, data, **kwargs):
+    def send(self, data):
         if not self.connected:
             return
         sent(f'{data}')
-        if not kwargs:
-            if self.type == "bot":
-                kwargs.update(self.bot_header())
-            elif self.type == "api":
-                kwargs.update(self.api_header())
-        stream = Stream.new(data, **kwargs)
+        from_addr = f"{self.client_address}"
+        to_addr = f"{self.server_address}"
+        stream = Stream(from_address=from_addr, to_address=to_addr, body=data)
         self.socket.sendall(stream.encode())
-        return Stream(stream=self.read(4096))
+        return self.read()
+
+    def ping(self):
+        if not self.connected:
+            return
+        from_addr = f"{self.client_address}"
+        to_addr = f"{self.server_address}"
+        stream = Stream(from_address=from_addr, to_address=to_addr, type='client', function='ping', body=self.server_id)
+        self.socket.sendall(stream.encode())
+        return self.read().body
+
 
     def __enter__(self):
         self.connect()
@@ -199,8 +164,8 @@ class GameBotClient:
     def __repr__(self):
         attr = {
             'connected': self.connected,
-            'host': self.server_host,
-            'port': self.server_port
+            'server_address': self.server_address,
+            'client_address': self.client_address
         }
         items = []
         for k, v in attr.items():
@@ -208,42 +173,4 @@ class GameBotClient:
         args = ', '.join(items)
         return f'<\033[96mGameBotClient\033[0m({args})>\033[0m'
 
-    def __del__(self):
-        message = f'Socket {self.__ID} Connection Ended'.center(100)
-        print(f'\r\033[7;35m{message}\033[0m')
 
-
-if __name__ == "__main__":
-    class A:
-        def __init__(self):
-            self.a = 1
-
-        def __getitem_(self, item):
-            return self.__dict__[item]
-
-        def keys(self):
-            return self.__dict__.keys()
-
-        def __str__(self):
-            return f"<A(a={repr(self.a)})>"
-
-        def __repr__(self):
-            return f"<A(a={repr(self.a)})>"
-
-if __name__ == "__main__":
-    with GameBotClient() as bot:
-        response = bot.send(A())
-
-    """
-from server.game_bot_client import GameBotClient
-with GameBotClient() as bot:
-    response = bot.send("Hello world!")
-    """
-
-    """
-cd ~/Ark-API && python3 -c "from server import GameBotClient; bot = GameBotClient(host='68.126.220.147'); bot.connect(); bot.disconnect()"; cd
-cd ~/Ark-API && python3 -c "from server import GameBotClient; bot = GameBotClient(host='172.20.176.1'); bot.connect(); bot.disconnect()"; cd
-cd ~/Ark-API && python3 -c "from server import GameBotClient; bot = GameBotClient(host='192.168.56.1'); bot.connect(); bot.disconnect()"; cd
-cd ~/Ark-API && python3 -c "from server import GameBotClient; bot = GameBotClient(host='192.168.1.135'); bot.connect(); bot.disconnect()"; cd
-cd ~/Ark-API && python3 -c "from server import GameBotClient; bot = GameBotClient(host='192.168.1.254'); bot.connect(); bot.disconnect()"; cd
-    """
